@@ -2,7 +2,8 @@
 
 #
 # dbroweval.pm
-# Copyright (C) 1991-2018 by John Heidemann <johnh@isi.edu>
+# Copyright (C) 1991-2008 by John Heidemann <johnh@isi.edu>
+# $Id$
 #
 # This program is distributed under terms of the GNU general
 # public license, version 2.  See the file COPYING
@@ -26,10 +27,10 @@ Evaluate code for each row of the data.
 Typical actions are things like reformatting
 and other data transformations.
 
-Code can include embedded column names preceded by underscores;
+Code can include embedded column names preceeded by underscores;
 these result in the value of that column for the current row.
 
-The values of the last row's columns are retrieved with _last_foo
+The values of the last row's columns are retreieved with _last_foo
 where foo is the column name.
 
 Even more perverse, _columname(N) is the value of the
@@ -64,17 +65,13 @@ no output at all, except for what is in the provided code
 =item B<-m> or B<--manual-output>
 
 The user must setup output,
-allowing arbitrary comments.
+allowing arbitary comments.
 See example 2 below for details.
 
 =item B<-w> or B<--warnings>
 
 Enable warnings in user supplied code.
 
-=item B<--saveoutput $OUT_REF>
-
-Save output writer (for integration with other fsdb filters).
-
 =back
 
 =for comment
@@ -105,11 +102,6 @@ but Fsdb::Filter objects in Perl do not run until you invoke
 the run() method.
 The C<--(no)autorun> option controls that behavior within Perl.
 
-=item B<--header> H
-
-Use H as the full Fsdb header, rather than reading a header from
-then input.
-
 =item B<--help>
 
 Show help.
@@ -124,32 +116,6 @@ Show full manual.
 end_standard_fsdb_options
 
 
-=head1 ADVANCED USAGE
-
-Typically L<dbroweval> outputs a line in the same schema for each input line.
-For advanced usage, one can violate each of these assumptions.
-
-Some fun:
-
-=over 4
-
-=item B<omitting a line>
-
-Add the code C<next row if ($your condition);>
-
-=item B<outputting an extra line>
-
-Call C<&$write_fastpath_sub($fref)>.
-You may find C<$fref>, the input row, useful.
-
-=item B<changing the schema>
-
-See the examples below in L</Command 2: Changing the Schema>
-
-=back
-
-
-
 =head1 SAMPLE USAGE
 
 =head2 Input:
@@ -213,34 +179,26 @@ Note that newly created columns I<do not> have underscore-names
 
 Thus a third equivalent is:
 
-    cat data.fsdb | \
-	dbroweval -m -b '@out_args = ( -clone => $in, \
-	         -cols => ($in->cols, divisible_by_ten); ' \
-	    'my $div_by_10 = (_size % 10 == 0); \
-	    $ofref = [ @$fref, $div_by_10 ] if ($div_by_ten);' |
+    cat data.fsdb |
+	dbroweval -m -b '@out_args = ( -clone => $in, -cols => ($in->cols, divisible_by_ten); ' \
+	    'my $div_by_10 = (_size % 10 == 0); $ofref = [ @$fref, $div_by_10 ] if ($div_by_ten);' |
 	dbcol size mean divisible_by_ten
 
 or
 
-    cat data.fsdb | \
-	dbroweval -m -b '@out_args = ( -clone => $in, \
-		-cols => [qw(size mean divisible_by_ten)] ); ' \
-	    'my $div_by_10 = (_size % 10 == 0);  \
-	    $ofref = [ _mean, _size, $div_by_10 ] if ($div_by_ten);'
+    cat data.fsdb |
+	dbroweval -m -b '@out_args = ( -clone => $in, -cols => [qw(size mean divisible_by_ten)] ); ' \
+	    'my $div_by_10 = (_size % 10 == 0); $ofref = [ _mean, _size, $div_by_10 ] if ($div_by_ten);'
 
 
 Finally, one can write different a completely different schema, although
 it's more work:
 
-    cat data.fsdb | \
-	dbroweval -m -b '@out_args = (-cols => [qw(size n)]);' \
-	    '$ofref = [ _size, 1 ];'
+    cat data.fsdb | dbroweval -m -b '@out_args = (-cols => [qw(size n)]);' '$ofref = [ _size, 1 ];'
 
 writes different columns, and
 
-    cat data.fsdb | \
-	dbroweval -n -m -b '@out_args = (-cols => [qw(n)]);  \
-	    my $count = 0;' -e '$ofref = [ $count ];' '$count++;'
+    cat data.fsdb | dbroweval -n -m -b '@out_args = (-cols => [qw(n)]); my $count = 0;' -e '$ofref = [ $count ];' '$count++;'
 
 Is a fancy way to count lines.
 
@@ -262,14 +220,6 @@ internal representation of the last row.
 Yes, this is a bit unappetizing, but,
 in for a penny with C<$ofref>, in for a pound.
 
-=head2 Command 4: Extra Ouptut
-
-Calling C<&$write_fastpath_sub($fref)> will do extra output,
-so this simple program will duplicate each line of input
-(one extra output, plus one regular output for each line of input):
-
-    dbroweval  '&$write_fastpath_sub($fref)'
-
 
 =head1 BUGS
 
@@ -290,7 +240,6 @@ L<Fsdb(3)>
 
 use strict;
 use Pod::Usage;
-use Carp;
 
 use Fsdb::Support;
 use Fsdb::Filter;
@@ -329,7 +278,6 @@ sub set_defaults ($) {
     $self->{_code_files} = [];
     $self->{_code_lines} = [];
     $self->{_warnings} = undef;
-    $self->{_header} = undef;
 
     $self->{_no_output} = undef;
     $self->{_no_output_even_comments} = undef;
@@ -367,14 +315,12 @@ sub parse_options ($@) {
 	'd|debug+' => \$self->{_debug},
 	'e|end=s@' => $self->{_end_code},
 	'f|code-files=s@' => $self->{_code_files},
-	'header=s' => \$self->{_header},
 	'i|input=s' => sub { $self->parse_io_option('input', @_); },
 	'log!' => \$self->{_logprog},
 	'm|manual-output' => \$self->{_manual_output}, 
 	'n|no-output' => \$self->{_no_output}, 
 	'N|no-output-even-comments' => \$self->{_no_output_even_comments},
 	'o|output=s' => sub { $self->parse_io_option('output', @_); },
-	'saveoutput=s' => \$self->{_save_output},
         'w|warnings!' => \$self->{_warnings},
 	) or pod2usage(2);
     # rest is code
@@ -398,7 +344,7 @@ sub setup ($) {
     # handle files
     #
     foreach (@{$self->{_code_files}}) {
-	open(INF, "<$_") || croak($self->{_prog} . ": cannot open ``$_''.\n");
+	open(INF, "<$_") || die $self->{_prog} . ": cannot open ``$_''.\n";
 	push(@{$self->{_code_lines}}, _confirm_ending_semicolon(join('', <INF>)));
 	close INF;
     };
@@ -407,14 +353,9 @@ sub setup ($) {
     # set up reader
     #
     $self->{_out} = undef;
-    my @in_options = ();
-    if ($self->{_no_output_even_comments}) {
-	$self->{_no_output} = 1;
-	push(@in_options, -outputheader => 'never');
-    };
-    push(@in_options, (-comment_handler => $self->create_pass_comments_sub))
+    my @in_options;
+    @in_options = (-comment_handler => $self->create_pass_comments_sub)
 	if (!$self->{_no_output_even_comments});
-    push(@in_options, -header => $self->{_header}) if (defined($self->{_header}));
     $self->finish_io_option('input', @in_options);
     my $read_fastpath_sub = $self->{_in}->fastpath_sub();
 
@@ -436,15 +377,15 @@ sub setup ($) {
 	 $any_needs_lfref = 1 if ($this_needs_lfref);
 	 my($code) = join("\n", @{$iref->[$DB_CODE_A_F]});
 	 $any_needs_lfref = 1 if ($code =~ /lfref/);
-	 $self->{_pretty_args} .= " $iref->[$COMMAND_F] " .  
-			Fsdb::Support::shell_quote("{ " . Fsdb::Support::code_prettify(@{$iref->[$DB_CODE_A_F]}) . " }");
+	 $self->{_pretty_args} .= " $iref->[$COMMAND_F] { " .
+			Fsdb::Support::code_prettify(@{$iref->[$DB_CODE_A_F]}) . " }";
 	 print STDERR "$iref->[$TITLE_F]:\n$c\n" if ($self->{_debug});
     };
     exit 1 if ($self->{_debug} == 1);
 
     #
     # write the loop
-    # xxx: should be able to optimize away $lfref
+    # xxx: should be able to otpimize away $lfref
     #
     {
 	my $loop_sub;
@@ -458,7 +399,7 @@ sub setup ($) {
 		&$write_fastpath_sub($ofref) if (defined($ofref));
 	    ';
 	if ($self->{_manual_output}) {
-	    $row_output_code = 'if (defined($ofref)) { &$write_fastpath_sub($ofref); $ofref = undef; };';
+	    $row_output_code = '&$write_fastpath_sub($ofref) if (defined($ofref));';
 	} else {
 	    @out_args = (-clone => $self->{_in});
 	    $row_output_code = ($self->{_no_output}) ? '' :
@@ -485,9 +426,8 @@ sub setup ($) {
 	            ' . (defined($code) ? $code : '') . q'
 		    # end user MAINLINE CODE
 		    ' . ($any_needs_lfref ? q'
-                    ' : '') . $row_output_code . q'
-		} continue {
 		    $lfref = $fref;  # save for next pass
+                    ' : '') . $row_output_code . q'
 		};
 		' . $output_end_pre . q'
 		# begin user END CODE
@@ -499,7 +439,7 @@ sub setup ($) {
 	print "\nLOOP CODE:\n$loop_code\n"
 	    if ($self->{_debug} >= 2);
 	eval $loop_code;
-	$@ && croak($self->{_prog} . ":  eval error compiling user-provided code: $@.\n: CODE:\n$loop_code\n");
+	$@ && die $self->{_prog} . ":  eval error compiling user-provided code: $@.\n: CODE:\n$loop_code\n";
 	$self->{_loop_sub} = $loop_sub;
     };
     $self->{_beg_code_final} = $beg_code;
@@ -520,45 +460,21 @@ sub run ($) {
 }
 
 
-=head2 finish
-
-    $filter->finish();
-
-Internal: write trailer.
-
-=cut
-sub finish($) {
-    my($self) = @_;
-    return if ($self->{_no_output_even_comments});
-    $self->SUPER::finish();
-}
-
-
 =head2 compute_program_log
 
     $log = $filter->figure_program_log();
 
-Override compute_program_log to do pretty-printed arguments.
+Override compute_program_log to do pretty-printted arguments.
 
 =cut
 
 sub compute_program_log($) {
     my $self = shift @_;
 
-    my $log = "  | " . $self->{_prog} . $self->{_pretty_args};
+    my $log = "  | " . $self->{_prog} .
+				" " . $self->{_pretty_args};
     return $log;
 }
 
 
-
-=head1 AUTHOR and COPYRIGHT
-
-Copyright (C) 1991-2018 by John Heidemann <johnh@isi.edu>
-
-This program is distributed under terms of the GNU general
-public license, version 2.  See the file COPYING
-with the distribution for details.
-
-=cut
-
 1;
